/*****************************************************************************
    AmyEdit - A GTKMM LaTeX Editor
    Copyright (C) 2004-2005 Rob Page
    email:  page <dot> rob <at> gmail <dot> com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "EditTabs.hh"
#include "NewDialog.hh"
#include "FileDialog.hh"
#include "FindDialog.hh"
#include "ChoiceDialog.hh"
#include "AmyEdit.hh"
#include "SpellDialog.hh"

#include <gtksourceview/gtksourcebuffer.h>
#include <gtksourceviewmm/sourcelanguage.h>
#include <gtksourceviewmm/sourcelanguagesmanager.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>
#include <gtksourceview/gtksourceiter.h>

#include <sstream>
#include <fstream>
#include <aspell.h>
#include <string>

EditTabs::EditTabs(PrefStruct *prefstruct) : Gtk::Notebook()
{
  preferences=prefstruct;

  spell_config= new_aspell_config();
  aspell_config_replace(spell_config, "lang", preferences->language.c_str());
  aspell_config_replace(spell_config, "mode", "tex");

  set_show_tabs(true);
  set_show_border(true);
  set_tab_pos(Gtk::POS_TOP);
  set_scrollable(true);

  extern SigC::Signal0<void> signal_preferences_update;

  signal_switch_page().connect(sigc::mem_fun(*this, &EditTabs::on_switching_page));
  signal_preferences_update.connect(sigc::mem_fun(*this, &EditTabs::on_preferences_update));
}

EditTabs::~EditTabs()
{

}

void EditTabs::NewTab()
{
  NewDialog dialog;
  
  int result=dialog.run();
  if (result==Gtk::RESPONSE_OK)
    {
      
      Page *newpage= Gtk::manage(new Page(preferences, this));
   
      newpage->filename=Glib::ustring("");
      newpage->get_buffer()->set_text(dialog.getDoc());
      
      // This needed to guarantee set_modified is changed, 
      // and thus signal handler gets called.
      newpage->get_buffer()->set_modified(true);  
      newpage->get_buffer()->set_modified(false);
      newpage->get_child()->grab_focus();
      newpage->get_buffer()->place_cursor(newpage->get_buffer()->begin());
      set_current_page(page_num(*newpage));
    }
  else if (result==Gtk::RESPONSE_NONE)
    {
      Page *newpage= Gtk::manage(new Page(preferences, this));
      newpage->filename=Glib::ustring("");
      newpage->get_buffer()->set_modified(true);
      newpage->get_buffer()->set_modified(false);
      newpage->get_child()->grab_focus();
      newpage->get_buffer()->place_cursor(newpage->get_buffer()->begin());
      set_current_page(page_num(*newpage));
    }
      
}

void EditTabs::OpenTab()
{
  FileDialog dialog(Glib::ustring("Open File..."),Gtk::FILE_CHOOSER_ACTION_OPEN);
  
  int result=dialog.run();
  if (result==Gtk::RESPONSE_OK)
    {
      OpenTab(dialog.get_filename());
    }
}

void EditTabs::OpenTab(Glib::ustring filename, bool open_if_not_found)
{
  Glib::ustring temp = "";
  bool found = false;
  if (Glib::file_test(filename,Glib::FILE_TEST_EXISTS))
    {
      temp=Glib::file_get_contents(filename);
      found = true;
    }
  if (found || open_if_not_found)
    {
      try 
	{
	  Page *newpage= Gtk::manage(new Page(preferences, this));
	  
	  newpage->filename=filename;
	  newpage->get_buffer()->insert_at_cursor(temp);
	  
	  // Ensure modified toggles, so that tab labels get updated, etc.
	  newpage->get_buffer()->set_modified(true);
	  newpage->get_buffer()->set_modified(false);
	  
	  newpage->get_child()->grab_focus();
	  newpage->get_buffer()->place_cursor(newpage->get_buffer()->get_iter_at_line(0));

	  set_current_page(page_num(*newpage));
	  AmyEdit::instance()->add_recent_file(filename);

	}
      catch(Glib::FileError error)
	{
	  Gtk::MessageDialog mdialog(error.what());
	  mdialog.run();
	}
    }
  else
    {
      Gtk::MessageDialog mdialog("File Not Found");
      mdialog.run();
    }
}
    
bool EditTabs::SaveCurrentTab()
{
  Page *curtab = get_current_tab();
  if (!curtab)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to save at present.. Try having a tab open"));
      dialog.run();
      return false;
    }
  else
    {
      
      if (curtab->filename==Glib::ustring(""))
	{
	  return SaveCurrentTabAs();
	}
      else
	{
	  return WriteTab();
	}
    }
}

bool EditTabs::SaveCurrentTabAs()
{
  // Check there's a Tab
  Page *curtab = get_current_tab();
 
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to save at present.. Try having a tab open"));
      dialog.run();
      return false;
    }
  else
    {
      // Open Save As Dialog
      FileDialog dialog(Glib::ustring("Save File As..."),Gtk::FILE_CHOOSER_ACTION_SAVE);
      int result=dialog.run();
      if(result==Gtk::RESPONSE_OK)
	{
	  bool writefile = true;
	  int mode = access(dialog.get_filename().c_str(),F_OK);
	  if (!mode)
	    {
	      ChoiceDialog chdialog(Glib::ustring("File ")+dialog.get_filename()+Glib::ustring(" already exists, do you wish to overwrite?"));
	      int result=chdialog.run();
	      if(result!=Gtk::RESPONSE_OK)
		writefile=false;
				    
	    }

	  // Update internal data
	  if (writefile)
	    {
	      curtab->filename=dialog.get_filename();
	      // Set Tab and WriteFile
	      curtab->tablabel->set_text(TabNameFromFile(curtab->filename));
	      return WriteTab();
	    }
	  return false;
	}
    }
}

bool EditTabs::WriteTab()
{
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to save at present.. Try having a tab open, although i'm not sure this shouldn't possible so please try and reproduce this message and email me a bug report"));
      dialog.run();
      return false;
    }
  else
    {
      try
	{
	  Glib::RefPtr<Glib::IOChannel>outfile = Glib::IOChannel::create_from_file(curtab->filename,"w");
	  outfile->write(curtab->get_buffer()->get_text());
	  outfile->flush();
	  curtab->get_buffer()->set_modified(false);
	  AmyEdit::instance()->add_recent_file (curtab->filename);

	}
      catch(Glib::FileError error)
	{
	  Gtk::MessageDialog dialog(error.what());
	  dialog.run();
	  return false;
	}
    }
  
  
  return true;
}

Glib::ustring EditTabs::Compile()
{
  Glib::ustring filename = "";
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to markup at present.. You'll be needing a document for this one"));
      dialog.run();
    }
  else
    {
      // Save File!
      if (SaveCurrentTab())
	{
	  filename=Glib::path_get_basename(curtab->filename);
	  std::string cmd = preferences->latex_command;
	  std::string::size_type repint;

	  while ((repint=cmd.find("temp.tex",0))!=std::string::npos)
	    {
	      cmd.replace(repint,8,filename);
	    }

	  int exitstat;
	  std::string sout;
	  std::string serr;
	  std::string pwd = Glib::getenv("PWD");

	  chdir(Glib::path_get_dirname(curtab->filename).c_str());

	  Glib::spawn_command_line_sync(cmd,&sout,&serr,&exitstat);

	  chdir(pwd.c_str());

	  AmyEdit::instance()->set_output_text(sout+serr);

	  if (exitstat)
	    {
	      AmyEdit::instance()->pop_up_output();
	      filename="";
	    }
	  else
	    {
	      std::string outputext = preferences->output_file;
	      outputext.erase(0,4);
	      filename.replace(filename.size()-4,4,outputext);
	      filename = Glib::build_filename(Glib::path_get_dirname(curtab->filename),filename);
	    }
	}
    }

  return filename;
}

bool EditTabs::SaveTmpTex()
{
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to markup at present.. You'll be needing a document for this one"));
      dialog.run();
      return false;
    }
  else
    {
      try
	{
	  Glib::RefPtr<Glib::IOChannel>outfile = Glib::IOChannel::create_from_file(Glib::get_home_dir()+"/.amyedit/temp.tex","w");
	  outfile->write(curtab->get_buffer()->get_text());
	  outfile->flush();
	  return true;
	}
      catch(Glib::FileError error){
	Gtk::MessageDialog dialog(error.what());
	dialog.run();
	return false;
      }
    }      
}

void EditTabs::SaveAllTabs()
{
  int cur=get_current_page();

  for (int x=0;x<get_n_pages();x++)
    {
      set_current_page(x);
      SaveCurrentTab();
    }

  set_current_page(cur);
}

bool EditTabs::CloseTab(int tab)
{
  extern SigC::Signal1<void,bool> signal_file_open;

  if (tab<0)
    tab=get_current_page();
  Page *curtab = get_current_tab();
  if (curtab!=NULL)
    {
      if (curtab->get_buffer()->get_modified())
	{
	  
	  ChoiceDialog dialog(curtab->tablabel->get_text()+Glib::ustring(" is unsaved!\nDo you wish to save it?"), YES_NO);
	  int close=dialog.run();
	  switch (close)
	    {
	    case Gtk::RESPONSE_YES:
	      SaveCurrentTab();
	      break;
	    case Gtk::RESPONSE_NO:

	      delete(get_nth_page(tab));
	      if (get_n_pages()==0)
		{
		  SetWindowTitle();
		  signal_file_open(false);
		}
	      break;
	    default:
	      return false;
	      break;
	    }
	}
      else
	{
	  delete(get_nth_page(tab));
	  if (get_n_pages()==0)
	    {
	      SetWindowTitle();
	      signal_file_open(false);
	    }
	}
    }
  else
    {
      Gtk::MessageDialog dialog("Nothing to close");
      dialog.run();
    }
  return true;
}

bool EditTabs::CloseAllTabs(){
  int cur=get_current_page();
  int max=get_n_pages();
  
  for (int x=0;x<max;x++)
    {
      set_current_page(0);
     
      if (!CloseTab())
	{
	  set_current_page(cur);
	  return false;
	}

    }

  set_current_page(cur);
  return true;
}  

Glib::ustring EditTabs::TabNameFromFile(Glib::ustring start)
{
  start.erase(0,start.rfind('/',start.length())+1);
  return (start);


}

void EditTabs::SaveCurrentTabAsTemplate()
{
  Gtk::Dialog savedialog;
  // Create a dialog with a text entry, to save template as..
  savedialog.set_title("Save Template...");
  Gtk::VBox *vbox=savedialog.get_vbox();
  Gtk::Entry templatename;
  
  vbox->add(*Gtk::manage(new class Gtk::Label("Enter Name:")));
  vbox->add(templatename);

  savedialog.add_button(Gtk::Stock::OK,Gtk::RESPONSE_OK);
  savedialog.add_button(Gtk::Stock::CANCEL,Gtk::RESPONSE_CANCEL);

  savedialog.show_all_children();
  if (savedialog.run() == Gtk::RESPONSE_OK)
    {
      std::ofstream outfile((Glib::ustring(getenv("HOME")) + "/.amyedit/templates/"+templatename.get_text()).c_str());
      if (!outfile)
	{
	  Gtk::MessageDialog dialog("Cannot open ~/.amyedit/templates/"+templatename.get_text()+" to save configuration\nThis could be a permissions error");
	  dialog.run();
	}
      else
	{
	  outfile << get_current_tab()->get_buffer()->get_text();
	}
      outfile.close();
    }


}
Glib::ustring EditTabs::GetUntitledTabName(Glib::ustring start)
{
  int p = get_n_pages();
  int i = 1;
  Glib::ustring name=start;
  bool exists=false;
  do{
    exists=false;
    // Horrible Hack to convert int to string
    std::ostringstream ostr;
    ostr << i;
    Glib::ustring str = ostr.str();
    
    name=start+str;
    
    for (int x=0;x<p;x++)
      {
	Page *curtab = static_cast<Page*>(get_nth_page(x));
	
	if (name==curtab->tablabel->get_text())
	  exists=true;
      }

	i++;
  }while(exists);

  return name;
}

void EditTabs::Undo()
{
  Glib::RefPtr<Gtk::SourceBuffer> buffer = get_current_view()->get_buffer();
     if (buffer->can_undo ())
       buffer->undo ();
       
}


void EditTabs::Redo()
{
     Glib::RefPtr<Gtk::SourceBuffer> buffer = get_current_view()->get_buffer();
     if (buffer->can_redo ())
       buffer->redo ();
       

}

void EditTabs::CutText()
{
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to cut at present.. Try having a tab open"));
      dialog.run();
    }
  else
    {
     
      Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
      curtab->get_buffer()->cut_clipboard(refClipboard);
      
    }

}

void EditTabs::CopyText()
{
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to copy at present.. Try having a tab open"));
      dialog.run();
    }
  else
    {
     
      Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
      curtab->get_buffer()->copy_clipboard(refClipboard);
      
    }
}

void EditTabs::PasteText()
{
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to paste at present.. Try having a tab open"));
      dialog.run();
    }
  else
    {
     
      Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
      curtab->get_buffer()->paste_clipboard(refClipboard);
      
    }

}

void EditTabs::DeleteText()
{
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to delete at present.. Try having a tab open"));
      dialog.run();
    }
  else
    {
     
      Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
      curtab->get_buffer()->erase_selection();
      
    }

}

void EditTabs::FindNext()
{
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to find on at present.. Try having a tab open"));
      dialog.run();
    }
  else
    {
      Glib::RefPtr<Gtk::SourceBuffer::Mark> cursor=curtab->get_buffer()->get_insert();
      Gtk::SourceBuffer::iterator start, end;
      GtkTextIter climit;
      Gtk::SourceBuffer::iterator limit(&climit);
      FindDialog fdialog;
      int result;
      do
	{

	  result=fdialog.run();
	  if (result==Gtk::RESPONSE_OK)
	    {
	      /* 
		 Older versions of Gtk::TextIter::forward_search
		 are buggy, use the C version to prevent lots of 
		 Gtk Warnings 
		 FIXME! Add a check in configure, to define the *better*
		 version if it is available 
	      */

	      if(gtk_source_iter_forward_search(cursor->get_iter().gobj(),fdialog.get_text().c_str(),fdialog.get_flags(),start.gobj(),end.gobj(),0))
		{
		  curtab->get_buffer()->select_range(end, start);
		  (static_cast<Gtk::SourceView*>(curtab->get_child()))->scroll_to(end);
		}
	      else
		{
		  Gtk::MessageDialog dialog(fdialog.get_text()+" not found!");
		  dialog.run();
		}
	      
	    }
	}while(result==Gtk::RESPONSE_OK );
    }
}

void EditTabs::SpellCheck()
{
  Page *curtab = get_current_tab();
  if (!curtab)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears that we are unable to process your request due to a lack of witches, please burn and try again"));
      dialog.run();
    }
  else
    {
      SpellDialog dialog(spell_config);
      Glib::RefPtr<Gtk::SourceBuffer> buffer=get_current_view()->get_buffer();
      
      buffer->set_check_brackets(false);

      Gtk::TextIter wordstart = buffer->begin();
      Gtk::TextIter wordend;
      Gtk::TextIter temp;
      gunichar c = wordstart.get_char();


      /* Skip to first non-whitespace character */
      while (c==' ' || c=='\n')
	{
	  wordstart++;
	  c=wordstart.get_char();
	}
  
      /* SpellCheck the Document */
      while (wordstart)
	{
	  wordend=wordstart;

	  c=wordend.get_char();
	  while (c!=' ' && wordend)
	    {
	      wordend++;
	      c=wordend.get_char();
	    }

	  dialog.setCheckString(wordstart.get_text(wordend));

	  AspellToken miss = dialog.getNextMiss();
	  
	  while(miss.len != 0)
	    {
	      
	      Gtk::TextIter tmpstart, tmpend;
	      tmpstart=wordstart;
	      tmpstart.forward_chars(miss.offset);
	      tmpend=tmpstart;
	      tmpend.forward_chars(miss.len);

	      buffer->select_range(tmpstart,tmpend);
	      get_current_view()->scroll_to(tmpstart, 0.1);

	      int res=dialog.run();
	      
	      if (res==Gtk::RESPONSE_NO)
		{
		  
		}
	      else if(res==Gtk::RESPONSE_YES)
		{
		  buffer->select_range(wordstart,wordend);
		  buffer->erase_selection();
		  buffer->insert_at_cursor(dialog.ret);
		  wordstart=buffer->get_insert()->get_iter();
		  wordend=buffer->get_insert()->get_iter();
		}
	      else
		{
		  buffer->place_cursor(wordend);
		  buffer->set_check_brackets();
		  return;
		}
	      miss = dialog.getNextMiss();
	      
	    }
	  wordstart= wordend;
	  c=wordstart.get_char();
	  while (wordstart && c==' ')
	    {
	      wordstart++;
	      c=wordstart.get_char();
	    }
  
	}

      if (!wordstart)
	{
	  Gtk::MessageDialog okdialog("SpellCheck Complete!");
	  okdialog.run();
	}
      buffer->place_cursor(wordstart);
      get_current_view()->scroll_to(wordstart, 0.1);

      buffer->set_check_brackets(true);

    }

}



Glib::ustring EditTabs::GetRootFileName()
{
  Page *curtab = get_current_tab();
  if (curtab==NULL)
    {
      Gtk::MessageDialog dialog(Glib::ustring("It appears you have nothing to paste at present.. Try having a tab open"));
      dialog.run();
      return "";
    }
  else
    {
	  return curtab->filename;
    }
}



void EditTabs::on_switching_page(GtkNotebookPage *page, guint number)
{
    extern SigC::Signal1<void, int> signal_line_set;
    extern SigC::Signal1<void, bool> signal_file_open;
    Page *curtab = get_current_tab();
    
    if (curtab!=NULL)
      {
	signal_line_set(curtab->get_buffer()->get_insert()->get_iter().get_line());
	curtab->get_child()->grab_focus();
	SetWindowTitle();
	signal_file_open(true);
	
      }

}


EditTabs::Page::Page(PrefStruct *prefstruct, EditTabs *parent){

  /* Note to self! CLEAN THIS UP!!!! */
  int w,h;

  preferences=prefstruct;

  set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS);
  Gtk::SourceView *sourceview=Gtk::manage(new Gtk::SourceView());

  sourceview->modify_font(Pango::FontDescription("monospace"));

  sourceview->set_wrap_mode(Gtk::WRAP_WORD_CHAR);
  add(*sourceview);

  on_preferences_update();

  extern SigC::Signal0<void> signal_preferences_update;
  get_buffer()->signal_modified_changed().connect(sigc::mem_fun(*this, &EditTabs::Page::on_modified_changed));
  get_buffer()->signal_changed().connect(sigc::mem_fun(*this, &EditTabs::Page::on_changed));
  get_buffer()->signal_mark_set().connect(sigc::mem_fun(*this, &EditTabs::Page::on_mark_changed));
  signal_preferences_update.connect(sigc::mem_fun(*this, &EditTabs::Page::on_preferences_update));

  /***********************
   * Syntax Highlighting *
   ***********************/

  Glib::RefPtr<Gtk::SourceBuffer> buffer = get_buffer();
  Glib::ustring mime_type("text/x-tex");

  Gtk::SourceLanguage* language = buffer->get_languages_manager()->get_language_from_mime_type(mime_type);

  
  if (language) 
    buffer->set_language(language);

  /*********************
   * The Tab Widget
   *********************/

  Gtk::HBox *tabwidget=Gtk::manage(new class Gtk::HBox());
  tablabel=Gtk::manage(new class Gtk::Label());

  Gtk::Button *tabbutton=Gtk::manage(new class Gtk::Button());
  tabbutton->add(*Gtk::manage(new class Gtk::Image(Gtk::StockID("gtk-close"), Gtk::IconSize(Gtk::ICON_SIZE_MENU))));
  Gtk::IconSize::lookup(Gtk::ICON_SIZE_MENU,w,h);
  
  tabbutton->set_size_request(w+2,h+2);
  tabbutton->set_relief(Gtk::RELIEF_NONE);
  tabbutton->signal_clicked().connect(sigc::mem_fun(*this,&EditTabs::Page::on_close_me));

  tabwidget->add(*tablabel);
  tabwidget->pack_start(*tabbutton,Gtk::PACK_SHRINK);
  tabwidget->show_all_children();

  /* end tab widget */

  parent->append_page(*this,*tabwidget);
  parent->show_all_children();

}

void EditTabs::Page::on_close_me(){
  EditTabs *parent = get_parent();
  parent->CloseTab(parent->page_num(*this));
}

void EditTabs::Page::on_modified_changed(){
  EditTabs *parent = get_parent();
  if (filename=="")
    {
       if (get_buffer()->get_modified())
	tablabel->set_text(parent->GetUntitledTabName("[Modified] "));
      else
	tablabel->set_text(parent->GetUntitledTabName("[Untitled] "));
    }
  else
    {
      if (get_buffer()->get_modified())
	tablabel->set_text(parent->TabNameFromFile(filename)+"*");
      else
	tablabel->set_text(parent->TabNameFromFile(filename));
    }
  parent->SetWindowTitle();
}


void EditTabs::Page::on_changed()
{
  extern SigC::Signal1<void, int> signal_line_set;
  signal_line_set(get_buffer()->get_insert()->get_iter().get_line());

}

void EditTabs::Page::on_mark_changed(Gtk::SourceBuffer::iterator iter, Glib::RefPtr<Gtk::SourceBuffer::Mark> mark){
  if (mark->get_name()=="insert")
    {
    extern SigC::Signal1<void, int> signal_line_set;

    signal_line_set(get_buffer()->get_insert()->get_iter().get_line());
    
    }
}

EditTabs *EditTabs::Page::get_parent()
{
  return (static_cast<EditTabs*>(Gtk::Widget::get_parent()));
}

Gtk::SourceView* EditTabs::Page::get_child()
{
  return (static_cast<Gtk::SourceView*>(Gtk::Bin::get_child()));
}

Glib::RefPtr<Gtk::SourceBuffer> EditTabs::Page::get_buffer()
{
  return (get_child()->get_buffer());
}

EditTabs::Page *EditTabs::get_current_tab()
{
  return (static_cast<EditTabs::Page*>(get_nth_page(get_current_page())));
}

Gtk::SourceView *EditTabs::get_current_view()
{
  return (static_cast<Gtk::SourceView*>(get_current_tab()->get_child()));
}


void EditTabs::Page::on_preferences_update()
{
  // For things that need to happen for all pages
  get_child()->set_tabs_width(preferences->tab_size);
  
  get_buffer()->set_highlight(preferences->syntax_highlighting);


}


void EditTabs::on_preferences_update()
{
  // For things that need to happen for the EditTabs object
  aspell_config_replace(spell_config, "lang", preferences->language.c_str());

}

void EditTabs::SetWindowTitle()
{
  EditTabs::Page* curtab=get_current_tab();
  if (curtab==NULL)
    {
      get_parent_window()->set_title(Glib::ustring("AmyEdit ")+VERSION);
    }
  else
    {
      get_parent_window()->set_title(Glib::ustring("AmyEdit ")+VERSION+" - " + curtab->tablabel->get_text());
    }
}
