/*****************************************************************************
    AmyEdit - A GTKMM LaTeX Editor
    Copyright (C) 2004-2005 Rob Page
    email:  page <dot> rob <at> gmail <dot> com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "SpellDialog.hh"

#include <gtkmm/table.h>
#include <gtkmm/stock.h>
#include <gtkmm/label.h>
#include <gtkmm/entry.h>
#include <gtkmm/liststore.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/button.h>

#include <aspell.h>


class ListCols : public Gtk::TreeModel::ColumnRecord
{
public:

  ListCols()
  { add(col); }
  
  Gtk::TreeModelColumn<Glib::ustring> col;
}columns;

SpellDialog::SpellDialog(AspellConfig *config) 
{
  set_title(_("Spell Check"));
  list =Gtk::ListStore::create(columns);
  alternatives.set_model(list);

  speller = 0;
  AspellCanHaveError * possible_err = new_aspell_speller(config);
  if (aspell_error_number(possible_err) != 0)
    puts(aspell_error_message(possible_err));
  else
    speller = to_aspell_speller(possible_err);

  checker = 0;
  possible_err = new_aspell_document_checker(speller);
  if (aspell_error_number(possible_err) != 0)
    puts(aspell_error_message(possible_err));
  else
    checker = to_aspell_document_checker(possible_err);

  aspell_document_checker_reset(checker);

  misslabel.set_sensitive(false);

  Gtk::Table *table=Gtk::manage(new class Gtk::Table(6,4));
  table->set_spacings(5);
  table->set_homogeneous();

  Gtk::Button *addbutton = Gtk::manage(new class Gtk::Button(_("Add")));
  Gtk::Button *checkbutton = Gtk::manage(new class Gtk::Button(_("Check")));
  Gtk::Button *skipbutton = Gtk::manage(new class Gtk::Button(_("Skip")));
  Gtk::Button *skipallbutton = Gtk::manage(new class Gtk::Button(_("Skip All")));
  Gtk::Button *replacebutton = Gtk::manage(new class Gtk::Button(_("Replace")));
  Gtk::Button *replaceallbutton = Gtk::manage(new class Gtk::Button(_("Replace All")));
  
  table->attach(*Gtk::manage(new class Gtk::Label(_("Word:"))), 0,1,0,1);
  table->attach(misslabel,1,3,0,1);
  table->attach(*addbutton,3,4,0,1);

  table->attach(*Gtk::manage(new class Gtk::Label(_("Replace With:"))), 0,1,1,2);
  table->attach(replaceentry,1,3,1,2);
  table->attach(*checkbutton,3,4,1,2);

  Gtk::ScrolledWindow *altwindow = Gtk::manage(new class Gtk::ScrolledWindow());
  altwindow->set_policy(Gtk::POLICY_NEVER,Gtk::POLICY_ALWAYS);
  altwindow->add(alternatives);
  altwindow->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
  table->attach(*altwindow,0,2,2,6);

  table->attach(*skipbutton,2,3,3,4);
  table->attach(*skipallbutton,3,4,3,4);
  table->attach(*replacebutton,2,3,4,5);
  table->attach(*replaceallbutton,3,4,4,5);

  get_vbox()->pack_start(*table,Gtk::PACK_EXPAND_WIDGET,5);
  add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CANCEL);

  alternatives.get_selection()->signal_changed().
    connect(sigc::mem_fun(*this, &SpellDialog::on_selection_changed));

  addbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_add));
  checkbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::get_suggestions));
  skipbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_skip));
  skipallbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_skipall));
  replacebutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_replace));
  replaceallbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_replaceall));

  show_all_children();
}

SpellDialog::~SpellDialog()
{
  aspell_speller_save_all_word_lists(speller);
  delete_aspell_speller(speller);
  delete_aspell_document_checker(checker);
}

void SpellDialog::setCheckString(Glib::ustring word)
{

  current_string=word;
  aspell_document_checker_process(checker, word.c_str(), -1);

}

AspellToken SpellDialog::getNextMiss()
{
  AspellToken miss=aspell_document_checker_next_misspelling(checker);

  if (miss.len == 0)
    {
      return miss;
    }
  else
    {
      Glib::ustring missword (current_string,miss.offset,miss.len);
      misslabel.set_text(missword);
      replaceentry.set_text(missword);

      return miss;

    }
}

int SpellDialog::run()
{
  if (skiprep.find(misslabel.get_text())!=skiprep.end())
    {
      ret=skiprep[misslabel.get_text()];
      if (ret=="")
	return (Gtk::RESPONSE_NO);
      else
	return (Gtk::RESPONSE_YES);
      
    }

  misslabel.set_text(misslabel.get_text());
  get_suggestions();
  Gtk::TreeModel::iterator iter = list->children().begin();
  if(iter)
    alternatives.get_selection()->select(iter);

  return Gtk::Dialog::run();
}

void SpellDialog::on_selection_changed()
{
  Gtk::TreeModel::iterator iter = alternatives.get_selection()->get_selected();
  if(iter)
    {
      Gtk::TreeModel::Row row = *iter;
      replaceentry.set_text(row[columns.col]);
    }
}

void SpellDialog::on_replace(){
  aspell_speller_store_replacement(speller, misslabel.get_text().c_str(), -1,
                               replaceentry.get_text().c_str(), -1);
  ret=replaceentry.get_text();
  response(Gtk::RESPONSE_YES);
}

void SpellDialog::on_replaceall(){
  aspell_speller_store_replacement(speller, misslabel.get_text().c_str(), -1,
                               replaceentry.get_text().c_str(), -1);
  skiprep[misslabel.get_text()]=replaceentry.get_text();
  ret=replaceentry.get_text();
  response(Gtk::RESPONSE_YES);
}

void SpellDialog::on_skip(){
  response(Gtk::RESPONSE_NO);
}

void SpellDialog::on_skipall(){
  skiprep[misslabel.get_text()]=Glib::ustring("");
  response(Gtk::RESPONSE_NO);
}

void SpellDialog::on_add(){
  aspell_speller_add_to_personal(speller, misslabel.get_text().c_str(), -1);
  response(Gtk::RESPONSE_NO);
}

void SpellDialog::get_suggestions(){
  Glib::ustring word=replaceentry.get_text();
  alternatives.remove_all_columns();
  list->clear();
  const AspellWordList * suggestions = aspell_speller_suggest(speller,word.c_str(), -1);
  AspellStringEnumeration * elements = aspell_word_list_elements(suggestions);
  const char * words;
  while ((words = aspell_string_enumeration_next(elements)) != NULL )
    {
      Gtk::TreeModel::Row row = *(list->append());
      row[columns.col] = words;
      
    }
  delete_aspell_string_enumeration(elements);
  
  alternatives.append_column("Alternatives", columns.col);
      
  alternatives.show_all_children();

}
